@@ -684,8 +684,15 @@ export class BlockContext implements PartialParse {
 
     let {line} = this
     for (;;) {
-      while (line.depth < this.stack.length) this.finishContext()
-      for (let mark of line.markers) this.addNode(mark.type, mark.from, mark.to)
+      for (let markI = 0;;) {
+        let next = line.depth < this.stack.length ? this.stack[this.stack.length - 1] : null
+        while (markI < line.markers.length && (!next || line.markers[markI].from < next.end)) {
+          let mark = line.markers[markI++]
+          this.addNode(mark.type, mark.from, mark.to)
+        }
+        if (!next) break
+        this.finishContext()
+      }
       if (line.pos < line.text.length) break
       // Empty line
       if (!this.nextLine()) return this.finish()
@@ -1353,7 +1360,7 @@ function elt(type: Type, from: number, to: number, children?: readonly (Element
   return new Element(type, from, to, children)
 }
 
-const enum Mark { Open = 1, Close = 2 }
+const enum Mark { None = 0, Open = 1, Close = 2 }
 
 /// Delimiters are used during inline parsing to store the positions
 /// of things that _might_ be delimiters, if another matching
@@ -1453,7 +1460,7 @@ const DefaultInline: {[name: string]: (cx: InlineContext, next: number, pos: num
     let canOpen = leftFlanking && (next == 42 || !rightFlanking || pBefore)
     let canClose = rightFlanking && (next == 42 || !leftFlanking || pAfter)
     return cx.append(new InlineDelimiter(next == 95 ? EmphasisUnderscore : EmphasisAsterisk, start, pos,
-                                         (canOpen ? Mark.Open : 0) | (canClose ? Mark.Close : 0)))
+                                         (canOpen ? Mark.Open : Mark.None) | (canClose ? Mark.Close : Mark.None)))
   },
 
   HardBreak(cx, next, start) {
@@ -1496,7 +1503,7 @@ const DefaultInline: {[name: string]: (cx: InlineContext, next: number, pos: num
         // Set any open-link markers before this link to invalid.
         if (part.type == LinkStart) for (let j = 0; j < i; j++) {
           let p = cx.parts[j]
-          if (p instanceof InlineDelimiter && p.type == LinkStart) p.side = 0
+          if (p instanceof InlineDelimiter && p.type == LinkStart) p.side = Mark.None
         }
         return link.to
       }
@@ -1633,7 +1640,7 @@ export class InlineContext {
   /// or both. Returns the end of the delimiter, for convenient
   /// returning from [parse functions](#InlineParser.parse).
   addDelimiter(type: DelimiterType, from: number, to: number, open: boolean, close: boolean) {
-    return this.append(new InlineDelimiter(type, from, to, (open ? Mark.Open : 0) | (close ? Mark.Close : 0)))
+    return this.append(new InlineDelimiter(type, from, to, (open ? Mark.Open : Mark.None) | (close ? Mark.Close : Mark.None)))
   }
 
   /// Add an inline element. Returns the end of the element.
